/*

This file contains sample games 1, 2, & 3, otherwise known as 
Arena, Double Arena, and Eternal Arena.  These demonstrate some 
basics of how to create a Game type in Star Control: TimeWarp

*/


/* 
TimeWarp Sample Game 1: Arena
This demonstrates how to create a new Game type

For this game, we'll just have a bunch of ships fighting 
the human player.  
*/

//first, we #include the files we need.  library headers go first, then melee.h

#include <allegro.h> //allegro library header

#include "../melee.h"          //used by all TW source files.  well, mostly.  
REGISTER_FILE                  //done immediately after #including melee.h, just because I said so
#include "../melee/mgame.h"    //Game stuff
#include "../melee/mcbodies.h" //asteroids & planets
#include "../melee/mship.h"    //ships
#include "../melee/mlog.h"     //networking / demo recording / demo playback
#include "../melee/mcontrol.h" //controllers & AIs
#include "../melee/mview.h"    //Views & messages
#include "../melee/mshppan.h"  //ship panels...


class SampleGame1 : public Game { //declaring our game type
	virtual void init(Log *_log) ; //happens when our game is first started
	};

void SampleGame1::init(Log *_log) {
	//you need to call Game::init very early on, to set stuff up... rarely do you want to do anything before that
	Game::init(_log);

	//prepare needs to be called before you add items, or do physics or graphics or anything like that.  Don't ask why.  
	prepare(); 

	// declare an integer we can use for whatever we might need an integer for
	int i; 
	for (i = 0; i < 3; i += 1) { //this causes the next stuff to happen 3 times
		addItem(new Asteroid()); //this adds an asteroid to the game
		} //so there will be 3 asteroids (asteroids automatically create new asteroids when they die...)

	//first, we create the teams.  This battle has just 2 sides; the human player, and the enemies
	TeamCode human, enemies;
	human = new_team();
	enemies = new_team();

	//this creates a keyboard controller, using configuration 0 by default, since we don't specify another configuration
	Control *c = create_control(channel_server, "Keyboard");
	//the first parameter is channel_server, which means that this player is on the server computer
	//the second parameter is "Keyboard", the name of a controller type

	//this creates a ship
	Ship *s = create_ship("tauhj", c, width/2, height/2, 0, human);
	//the 1st parameter is a 5-letter ship code.  e.g. andgu = Androsynth Guardian, chmav = Chmmr Avatar, estgu = Estion Gunner, tauhj = TauHJ, thrto = Thraddash Torch
	//the 2nd parameter is the controller for the ship.  
	//the 3th & 4th parameters are the x & y coordinates where it appears
	//the 5th parameter is the angle that the ship starts out facing (in degrees)
	//the 6th parameter is the team it's on... this parameter is optional... if we leave it out, then it's on it's own team of 1, and everyone is its enemy
	//there is no 7th parameter used here, but there is the option of putting an .ini file name there, to override the one that normally corresponds to that ship

	//this causes the ship to warp into the game.  note that we call addItem on the phaser, not the ship, because the phaser will add the ship once it finishes warping in
	addItem(s->get_ship_phaser());
	//if we wanted the ship to just appear without warping in, then we would say "addItem(s);" instead

	//this causes the human ship to be a focus for the camera
	add_focus(s);

	//now, we create some enemies :)

	Ship *e;
	e = create_ship(channel_none, "kzedr", "WussieBot", width/4, height/4, random()%360, enemies);
	//This is a different syntax for create_ship
	//It creates a ship and an AI for it at the same time.  
	//AIs created in this way are automatically destroyed when their ship dies
	//Notice how it takes the parameters normally taken by both a create_control and a create_ship
	//Anyway, this creates a Kzer-Za Dreadnought, and WussieBot to control it
	//This ship starts facing a random direction
	//Notice that it's on channel channel_none.  This mean that the AI is considered part of physics and uses no bandwidth in network games
	//channel_none may not work with all AIs.  You can also use a channel_server or channel_client, to locate the AI on the server or client computer.  
	addItem(e->get_ship_phaser());
	e = create_ship(channel_none, "kohma", "WussieBot", width*3/4, height/4, random()%360, enemies);
	//here we add a Kohr-Ah Marauder
	addItem(e->get_ship_phaser());
	e = create_ship(channel_none, "druma", "WussieBot", width*3/4, height*3/4, random()%360, enemies);
	//here we add a Druuge Mauler
	addItem(e->get_ship_phaser());
	e = create_ship(channel_none, "yehte", "WussieBot", width/4, height*3/4, random()%360, enemies);
	//here we add a Yehat Terminator
	addItem(e->get_ship_phaser());

	//BTW, this is a vicious combination of enemies.  Yehat for direct assault, Druuge for long range support, and the Urquans to make life miserable

	message.out("Welcome to Sample Game 1: Arena!", 3000, 15);
	//display a welcoming message to the player
	//the message is displayed for 3000 milliseconds (3 seconds)
	//in color 15 of the standard TW pallete, which is white

	}

REGISTER_GAME(SampleGame1, "Sample: Arena") //registering our game type, so that it will appear in the menus


/*
TimeWarp Sample 2: Double Arena!

The differences from sample game 1:
1.  2 players are allowed in network games.  
2.  Players are permitted to pick their own ships.  
3.  The ship panels for human players are displayed.  
4.  The map is slightly larger.  
5.  The player is forced to use the "Hero" view.  
*/


class SampleGame2 : public Game { //declaring our game type
	virtual void init(Log *_log) ; //happens when our game is first started
	virtual void set_resolution(int screen_x, int screen_y) ; //we want to change our screen layout, to make room for the ship panels
	ShipPanel *human_panel[2];  //we need to keep track of the ship panels, in case we have to move them around because the player changes screen resolutions
	virtual void calculate(int time) ; 
	//to do the screen layouts, we need to know if those panels still exist, so we need to check every frame for them going away
	//calculate is good for things that need to be done every frame
	};

void SampleGame2::calculate(int time) {
	if (human_panel[0] && !human_panel[0]->exists()) human_panel[0] = NULL;
	if (human_panel[1] && !human_panel[1]->exists()) human_panel[1] = NULL;
	//as soon as a panel starts to die, we must get rid of our reference to it
	//otherwise we might accidentally use a reference to something that has already ceased to exist
	//which would crash the game
	Game::calculate(time);
	return;
	}
void SampleGame2::set_resolution(int screen_x, int screen_y) {
	view->set_window(screen, //the view window is on the screen
		0, 0, //with the upper left hand corner of the view window on the upper left corner of the screen
		screen_x - PANEL_WIDTH, screen_y //the height is full-screen, but the width doesn't cover the right-hand side, to make room for the ship panel there
		);
	if (human_panel[0]) human_panel[0]->locate(screen_x - 64, 0);   //if the panel still exists, position on the right-hand side
	if (human_panel[1]) human_panel[1]->locate(screen_x - 64, 100); //same for the other panel, if there is another panel
	redraw();
	}
void SampleGame2::init(Log *_log) {
	Game::init(_log);

	width *= 1.2;
	height *= 1.2;
	//width & height are the dimensions of the game map
	//they should be modified before prepare() is called
	//If for some reason you modify them later, call prepare() again
	//but you probably shouldn't be modifying them later. 
	//The normal size is 3840x3840
	prepare(); 

	int i;
	for (i = 0; i < 7; i += 1) addItem(new Asteroid());

	TeamCode humans, enemies;
	humans = new_team();
	enemies = new_team();

	Control *c = create_control(channel_server, "Keyboard");

	//this is where we deviate from SampleGame1
	log_file("fleets/all.scf"); //this is a fleet file containing every ship (more or less)
	load_fleet(NULL, "Fleet");
	//to load a fleet from disk you say load_fleet(filename, fleetname); (fleetname is usually "Fleet", because that is the standard name used in .scf files)
	//but that's not network enabled, so we manually load the file using the network-enabled log_file(), and pass NULL for the file name
	//BTW, once a fleet is loaded, you can find out things about it like, the cost of the fleet is fleet_cost and the size is fleet_size
	i = c->choose_ship("Hey You!\nPick a ship!");
	//now i is the number of the ship the user selected from that list
	log_int(channel_server, i);
	//necessary for networking... because i would otherwise be different on the 2 computers...
	//channel_server is used instead of channel_client becaues we want the servers value to override the clients
	if (i == -1) i = random() % fleet_size;
	//handles the case where random or always random was used

	//now we're back to the way we were in SampleGame1 for a little bit
	Ship *s = create_ship(fleet_id[i], c, width/2 + 100, height/2, 0, humans);
	addItem(s->get_ship_phaser());

	add_focus(s, channel_server);
	//in SampleGame1, we just said "add_focus(s);" to make it a focus for the camera
	//but here it's different, because we don't want it to be a focus for all computers, only the server computer

	human_panel[0] = new ShipPanel(s);
	addPresence(human_panel[0]);

	if ((log->type == Log::log_net1server) || (log->type == Log::log_net1client)) { 
		//if we're in a network game...
		//then do the same thing, only for the other player
		Control *c = create_control(channel_client, "Keyboard");
		log_file("fleets/all.scf");
		load_fleet(NULL, "Fleet");
		i = c->choose_ship("Hey You!\nPick a ship!");
		log_int(channel_client, i);
		if (i == -1) i = random() % fleet_size;
		Ship *s = create_ship(fleet_id[i], c, width/2 + 100, height/2, 180, humans);
		addItem(s->get_ship_phaser());
		add_focus(s, channel_client);
		human_panel[1] = new ShipPanel(s);
		addPresence(human_panel[1]);
		}
	else human_panel[1] = NULL;

	//now, we need some enemies :)

	Ship *e;
	e = create_ship(channel_server, "kzedr", "WussieBot", width/4, height/4, random()%360, enemies);
	addItem(e->get_ship_phaser());
	e = create_ship(channel_server, "kohma", "WussieBot", width*3/4, height/4, random()%360, enemies);
	addItem(e->get_ship_phaser());
	e = create_ship(channel_server, "druma", "WussieBot", width*3/4, height*3/4, random()%360, enemies);
	addItem(e->get_ship_phaser());
	e = create_ship(channel_server, "yehte", "WussieBot", width/4, height*3/4, random()%360, enemies);
	addItem(e->get_ship_phaser());
	//note that all the enemy AIs are located on the server
	//you can also have AIs colocated
	//to do that, you pass channel_none
	//benefits: an AI that is colocated uses no network bandwidth
	//disadvantages: colocating an AI may not work with all AI types; if it doesn't work, it will cause desynchs
	//Of course, only AIs can be colocated; you can't colocate a keyboard or joystick control, or bad things will happen

	set_view(view2number("Hero")); 
	//sets it to Hero view mode
	view_locked = true;
	//prevents the view mode from being changed through the menu

	message.out("Welcome to Sample Game: Double Arena!", 3000, 15);
	}

REGISTER_GAME(SampleGame2, "Sample: Double Arena!")


/* 

TimeWarp Sample 3: Eternal Arena!

The differences from sample game 2:
1.  5 seconds after either all AIs or all humans die, the game restarts.  

*/




class SampleGame3 : public Game { //declaring our game type
	virtual void init(Log *_log) ; //happens when our game is first started
	virtual void set_resolution(int screen_x, int screen_y) ; //we want to change our screen layout, to make room for the ship panels
	ShipPanel *human_panel[2];  //we need to keep track of the ship panels, in case we have to move them around because the player changes screen resolutions
	Control *human_control[2];
	virtual void calculate(int time) ; 
	//to do the screen layouts, we need to know if those panels still exist, so we need to check every frame for them going away
	//calculate is good for things that need to be done every frame
	TeamCode human_team, enemy_team; //the two teams
	int respawn_time; //the time left until the game is restarted
	void pick_new_ships(); //restarts the game
	};

void SampleGame3::calculate(int time) {
	Game::calculate(time);
	if (human_panel[0] && !human_panel[0]->exists()) human_panel[0] = NULL;
	if (human_panel[1] && !human_panel[1]->exists()) human_panel[1] = NULL;
	if (respawn_time == -1) {
		int i, humans = 0, enemies = 0;
		for (i = 0; i < num_targets; i += 1) {
			if (target[i]->get_team() == human_team) humans += 1;
			if (target[i]->get_team() == enemy_team) enemies += 1;
			}
		//if either team has no targetable items remaining (generally ships), pick new ships
		if (!humans || !enemies) respawn_time = game_time + 5000; //5000 milliseconds is 10 seconds
		}
	else if (respawn_time <= game_time) pick_new_ships();
	return;
	}

void SampleGame3::set_resolution(int screen_x, int screen_y) {
	view->set_window(screen, //the view window is on the screen
		0, 0, //with the upper left hand corner of the view window on the upper left corner of the screen
		screen_x - 64, screen_y //the height is full-screen, but the width doesn't cover the right-hand side, to make room for the ship panel there
		);
	if (human_panel[0]) human_panel[0]->locate(view->view_x + view->view_w, 0);   //if the panel still exists, position on the right-hand side
	if (human_panel[1]) human_panel[1]->locate(view->view_x + view->view_w, 100); //same for the other panel, if there is another panel
	redraw();
	}

void SampleGame3::pick_new_ships() {
	int i;
	acquire_screen();
	clear_to_color(screen, pallete_color[4]);
	release_screen();
	for (i = 0; i < num_items; i += 1) {
		if (item[i]->exists()) item[i]->die();
		}
	if (human_panel[0]) human_panel[0]->hide();
	if (human_panel[1]) human_panel[1]->hide();

	//player 1 selects a new ship
	log_file("fleets/all.scf");
	load_fleet(NULL, "Fleet");
	i = human_control[0]->choose_ship("Hey You!\nPick a ship!");
	log_int(channel_server, i);
	if (i == -1) i = random() % fleet_size;
	Ship *s = create_ship(fleet_id[i], human_control[0], width/2 + 100, height/2, 0, human_team);
	addItem(s->get_ship_phaser());
	human_panel[0] = new ShipPanel(s);
	addPresence(human_panel[0]);
	add_focus(s, channel_server);

	if ((log->type == Log::log_net1server) || (log->type == Log::log_net1client)) { 
		log_file("fleets/all.scf");
		load_fleet(NULL, "Fleet");
		i = human_control[1]->choose_ship("Hey You!\nPick a ship!");
		log_int(channel_client, i);
		if (i == -1) i = random() % fleet_size;
		Ship *s = create_ship(fleet_id[i], human_control[1], width/2 + 100, height/2, 180, human_team);
		addItem(s->get_ship_phaser());
		add_focus(s, channel_client);
		human_panel[1] = new ShipPanel(s);
		addPresence(human_panel[1]);
		}
	else human_panel[1] = NULL;


	Ship *e;
	e = create_ship(channel_none, "kzedr", "WussieBot", width/4, height/4, random()%360, enemy_team);
	addItem(e->get_ship_phaser());
	e = create_ship(channel_none, "kohma", "WussieBot", width*3/4, height/4, random()%360, enemy_team);
	addItem(e->get_ship_phaser());
	e = create_ship(channel_none, "druma", "WussieBot", width*3/4, height*3/4, random()%360, enemy_team);
	addItem(e->get_ship_phaser());
	e = create_ship(channel_none, "yehte", "WussieBot", width/4, height*3/4, random()%360, enemy_team);
	addItem(e->get_ship_phaser());

	addItem(new Planet(width/4, 0, planetSprite, 0));

	if (human_panel[0]) human_panel[0]->locate(view->view_x + view->view_w, 0);   //if the panel still exists, position on the right-hand side
	if (human_panel[1]) human_panel[1]->locate(view->view_x + view->view_w, 100); //same for the other panel, if there is another panel

	redraw();
	respawn_time = -1;
	return;
	}

void SampleGame3::init(Log *_log) {
	Game::init(_log);

	width = 4200;
	height = 4200;
	//width & height are the dimensions of the game map
	//they should be modified before prepare() is called
	//If for some reason you modify them later, call prepare() again
	//but you probably shouldn't be modifying them later. 
	//The normal size is 3840x3840
	prepare();

	human_team = new_team();
	enemy_team = new_team();

	human_control[0] = create_control(channel_server, "Keyboard");
	
	if ((log->type == Log::log_net1server) || (log->type == Log::log_net1client)) { 
		human_control[1] = create_control(channel_client, "Keyboard");
		}
	else human_control[1] = NULL;

	int i;
	for (i = 0; i < 7; i += 1) addItem(new Asteroid());

	human_panel[0] = NULL;
	human_panel[1] = NULL;

	respawn_time = 500;

	message.out("Welcome to Sample Game 3: Eternal Again!", 3000, 15);
	}

REGISTER_GAME(SampleGame3, "Sample: Eternal Arena")