#ifndef _MFRAME_H
#define _MFRAME_H

extern int frame_time;
extern DATAFILE *melee;

typedef unsigned int TeamCode;

class Planet;
class Query;

class Presence { 
	public:
	enum {
		PASSIVE_SYNCHRONIZATION, ACTIVE_SYNCHRONIZATION, NO_SYNCHRONIZATION
		};
	char synchronization;
	char state;      // may be set to any positive value if alive, or 0 to kill it
	virtual void animate(Frame *space); //displays ? on screen (NOT permitted to affect game physics in any way)
	virtual void animate_predict(Frame *space, int time); //like animate, but attempts to predict the future
	virtual void calculate(); //advance the item frame_time milliseconds in time
	virtual int exists();  //returns 0 if dead, non-zero if alive
	Presence();
	virtual ~Presence();  // called when an presence is deallocated
	virtual SpaceLocation *get_focus();
//	void play_sound (SAMPLE *sample, int vol = 255, int freq = 1000); //plays a sound
//	void play_sound2 (SAMPLE *sample, int vol = 255, int freq = 1000);//like play_sound, only stops the previous sound
	};

class SpaceLocation : public Presence { // base class for all items in game
	friend Physics;
	protected:
	public:
	int    id;          // id code, indicates what type it is
	double x;
	double y;
	double angle;       // the angle it's facing (sometimes not very meaningfull)
	double vx;          // vx, vy = x & y components of the velocity vector
	double vy;
	public:
	int    layer;       // determines & collisions
	int    depth;       // determines rendering order
	void set_depth(double d);
	double get_depth();

	enum { 
			team_bits = 14, ship_bits =  18, 
			team_shift = 0, ship_shift = 14, 
			team_mask = ((1<<team_bits)-1) << team_shift,
			ship_mask = ((1<<ship_bits)-1) << ship_shift
			};
	unsigned int ally_flag;

	public:

	Ship *ship;          // the ship it's associated with
	ShipData *data;      // the data module it depends upon
	SpaceObject *target; // it's target, if it has one

	inline  bool sameShip (SpaceLocation *other) {return ally_flag == other->ally_flag;}
	virtual bool sameTeam (SpaceLocation *other);
	TeamCode get_team();

	virtual void ship_died();
	virtual void target_died();
	virtual bool change_owner(SpaceLocation *new_owner);


	int    collide_flag_sameship; // mask for which layers it can collide with objects associated with the same ship
	int    collide_flag_sameteam; // mask for which layers it can collide with for allies
	int    collide_flag_anyone;   // mask for which layers it can collide with for non-allies

	int    damage_factor;         // the damage this item inflicts

	int    damage;               // the damage suffered by this object but not yet processed
	int    directDamage;         // same as damage, but not blockable
	int    repair;               // the opposite of damage
	int    fuel_sap;             // like damage, but affects fuel
	double speed_loss;           // like damage, but affects speed

	virtual void handle_damage(SpaceLocation *source);// called after setting damage etc. with the source of the damage

	SpaceLocation(SpaceLocation *creator, double lx, double ly, double langle);
	virtual void death();      // called after an item is killed
	virtual ~SpaceLocation();  // called when an item is deallocated

	virtual bool die(); // calling this kills an item (returns true on success)

	double get_angle();        // various public functions for reading otherwise protected data
	double get_vx();
	double get_vy();

	int getID();

	virtual SpaceLocation *get_focus();

	double normal_x();         // like the above, but also doublechecks that x is in the normal range (0 <= x < MAX_X)
	double normal_y();
	double nearest_x(SpaceLocation *l);
	double nearest_y(SpaceLocation *l);
	double distance(SpaceLocation *l);
	double trajectory_angle(SpaceLocation *l);

	int isLine();      //returns true if it is the specified engine-defined type
	int isObject();
	int isAsteroid();
	int isPlanet();
	int isShip();
	int isShot();
	int isEntity();

	virtual int isProtected();  // returns 0 normally, or a positive number if shielded
	virtual int isInvisible();  // returns 0 normally, or a positive number if cloaked

	Planet *nearest_planet(); //returns the nearest planet, or NULL if no planets are nearby

	virtual int canCollide(SpaceLocation *other); // returns 0 if collision impossible
	union {
		SpaceObject   *qnexto;
		SpaceLine     *qnextl;
		SpaceLocation *qnext;
		};

	virtual int translate(double dx, double dy); //moves an object
	virtual int accelerate(SpaceLocation *source, double angle, double vel, double max_speed); //accelerates an object, to a maximum of max_speed
	virtual int accelerate_gravwhip(SpaceLocation *source, double angle, double velocity, double max_speed) ;

	virtual void animate(Frame *space); //displays an object on screen (NOT permitted to affect game physics in any way)
	virtual void animate_predict(Frame *space, int time); //like animate, but attempts to predict the future
	virtual void calculate(); //advance the item frame_time milliseconds in time

	void play_sound (SAMPLE *sample, int vol = 255, int freq = 1000); //plays a sound
	void play_sound2 (SAMPLE *sample, int vol = 255, int freq = 1000);//like play_sound, only stops the previous sound
	};

class SpaceObject : public SpaceLocation {
	public:
	double       w;    //width of sprite
	double       h;    //height of sprite
	double       mass; //mass of object
	protected:
	SpaceSprite *sprite; //the pictures that this object looks like
	int          sprite_index; //which one of those pictures is active at the moment

	public:
	SpaceObject(SpaceLocation *creator, double ox, double oy, double oangle, 
			SpaceSprite *osprite);

	SpaceSprite *getSprite();
	int getSpriteIndex();

	double width();
	double height();

	virtual void animate(Frame *space);
	virtual void collide(SpaceObject *other);
	virtual double collide_line(double lx1, double ly1, double lx2, double ly2,
			double llength);

	virtual void inflict_damage(SpaceObject *other);
	};


class SpaceLine : public SpaceLocation {
	protected:
	double length;
	int    color;

	public:
	SpaceLine(SpaceLocation *creator, double lx, double ly, double langle, 
		double llength, int lcolor);

	double edge_x();
	double edge_y();
	double get_length();

	virtual void animate(Frame *space);
	virtual void collide(SpaceObject *o);
	virtual void inflict_damage(SpaceObject *other);
	};

extern int frame_time;
class Physics {
	private:
	public:
	//SpaceLocation *last_deleted;
	SpaceLocation **quadrant;
	friend Query;
	int num_items, max_items;
	SpaceLocation **item;
	int num_presences, max_presences;
	Presence **presence;

	TeamCode last_team;
	unsigned int last_ship;

	protected:
	void collide();

	virtual unsigned int new_ship();
	virtual TeamCode new_team();
	unsigned int get_code(unsigned int ship, TeamCode team) ;
	virtual void merge_teams ( TeamCode team1, TeamCode team2) ; //causes team1 and team2 to be the same team
	virtual void switch_team ( unsigned int ship,  TeamCode team ) ; //causes ship to join team.  if ship was already on an old team, ship betrays that team.  

	public:
	int game_time;
	double width;
	double height;
	virtual int checksum();
	virtual void prepare();

	SpaceLocation *addItem(SpaceLocation *o);
	bool removeItem(SpaceLocation *o);
	Presence *addPresence(Presence *o) ;
	bool removePresence(Presence *o) ;

	virtual void calculate(int time);
	virtual void animate(Frame *frame);
	virtual void animate_predict(Frame *frame, int time);

	virtual void play_sound (SAMPLE *sample, SpaceLocation *source, int vol = 255, int freq = 1000);
	virtual void play_sound2 (SAMPLE *sample, SpaceLocation *source, int vol = 255, int freq = 1000);

	virtual void preinit();
	virtual void init();
	virtual ~Physics();
	};

class Query {
	int layers;
	SpaceLocation *target;
	int qx_min, qx_max;
	int qy_min, qy_max;
	int qx, qy;
	double tx, ty, range;
	void next_quadrant ();
	public:
	union {
		SpaceObject   *currento;
		SpaceLine     *currentl;
		SpaceLocation *current;
		};
	void begin (SpaceLocation *target, int layers, double range);
	void begin (double x, double y, int layers, double range);
	void next ();
	void end();
	private:
	bool current_invalid() {
		if (!(bit(current->layer) & layers) || (current == target) || !current->exists()) return true;
		if (distance_from(tx, ty, current->normal_x(), current->normal_y()) > range) return true;
		return false;
		}
	};

#endif